"""
Compatibility module for safe and sane User model import.
"""

__title__ = 'nine.user'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('User',)

import importlib

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from nine.versions import DJANGO_LTE_1_4, DJANGO_LTE_1_6, DJANGO_GTE_1_7

if DJANGO_LTE_1_4:
    from django.contrib.auth.models import User
else:
    # Cannot use contrib.auth.get_user_model() at compile time.
    user_app_name, user_model_name = settings.AUTH_USER_MODEL.rsplit('.', 1)
    User = None

    if DJANGO_LTE_1_6:
        for app in settings.INSTALLED_APPS:
            if app.endswith(user_app_name):
                user_app_models = importlib.import_module(app + ".models")
                User = getattr(user_app_models, user_model_name)
                break
    elif DJANGO_GTE_1_7:
        from django.apps import apps
        try:
            User = apps.get_registered_model(user_app_name, user_model_name)
        except KeyError:
            pass

    if User is None:
        raise ImproperlyConfigured(
            "You have defined a custom user model {0}, but the app {1} is not in "
            "``settings.INSTALLED_APPS``"
            "".format(settings.AUTH_USER_MODEL, user_app_name)
        )



__title__ = 'nine.tests.test_versions'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('VersionsTest',)

import unittest
import mock
# For Python3 >= 3.4
try:
    from importlib import reload
# For Python3 < 3.4
except ImportError as err:
    try:
        from imp import reload
    except ImportError as err:
        pass

import django

from nine.tests.base import print_info

class VersionsTest(unittest.TestCase):
    """
    Tests of ``nine.versions`` module.
    """
    def setUp(self):
        pass

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.4.21'))
    def test_django_1_4_21(self):
        """
        Tests as if we were using Django==1.4.21.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(versions.DJANGO_1_4)
        self.assertTrue(not versions.DJANGO_1_5)
        self.assertTrue(not versions.DJANGO_1_6)
        self.assertTrue(not versions.DJANGO_1_7)
        self.assertTrue(not versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)

        # Less than or equal matching
        self.assertTrue(versions.DJANGO_LTE_1_4)
        self.assertTrue(versions.DJANGO_LTE_1_5)
        self.assertTrue(versions.DJANGO_LTE_1_6)
        self.assertTrue(versions.DJANGO_LTE_1_7)
        self.assertTrue(versions.DJANGO_LTE_1_8)
        self.assertTrue(versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(not versions.DJANGO_GTE_1_5)
        self.assertTrue(not versions.DJANGO_GTE_1_6)
        self.assertTrue(not versions.DJANGO_GTE_1_7)
        self.assertTrue(not versions.DJANGO_GTE_1_8)
        self.assertTrue(not versions.DJANGO_GTE_1_9)

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.5.5'))
    def test_django_1_5_5(self):
        """
        Tests as if we were using Django==1.5.5.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(not versions.DJANGO_1_4)
        self.assertTrue(versions.DJANGO_1_5)
        self.assertTrue(not versions.DJANGO_1_6)
        self.assertTrue(not versions.DJANGO_1_7)
        self.assertTrue(not versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)

        # Less than or equal matching
        self.assertTrue(not versions.DJANGO_LTE_1_4)
        self.assertTrue(versions.DJANGO_LTE_1_5)
        self.assertTrue(versions.DJANGO_LTE_1_6)
        self.assertTrue(versions.DJANGO_LTE_1_7)
        self.assertTrue(versions.DJANGO_LTE_1_8)
        self.assertTrue(versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(versions.DJANGO_GTE_1_5)
        self.assertTrue(not versions.DJANGO_GTE_1_6)
        self.assertTrue(not versions.DJANGO_GTE_1_7)
        self.assertTrue(not versions.DJANGO_GTE_1_8)
        self.assertTrue(not versions.DJANGO_GTE_1_9)

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.6.5'))
    def test_django_1_6_5(self):
        """
        Tests as if we were using Django==1.6.5.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(not versions.DJANGO_1_4)
        self.assertTrue(not versions.DJANGO_1_5)
        self.assertTrue(versions.DJANGO_1_6)
        self.assertTrue(not versions.DJANGO_1_7)
        self.assertTrue(not versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)

        # Less than or equal matching
        self.assertTrue(not versions.DJANGO_LTE_1_4)
        self.assertTrue(not versions.DJANGO_LTE_1_5)
        self.assertTrue(versions.DJANGO_LTE_1_6)
        self.assertTrue(versions.DJANGO_LTE_1_7)
        self.assertTrue(versions.DJANGO_LTE_1_8)
        self.assertTrue(versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(versions.DJANGO_GTE_1_5)
        self.assertTrue(versions.DJANGO_GTE_1_6)
        self.assertTrue(not versions.DJANGO_GTE_1_7)
        self.assertTrue(not versions.DJANGO_GTE_1_8)
        self.assertTrue(not versions.DJANGO_GTE_1_9)

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.7.5'))
    def test_django_1_7_5(self):
        """
        Tests as if we were using Django==1.7.5.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(not versions.DJANGO_1_4)
        self.assertTrue(not versions.DJANGO_1_5)
        self.assertTrue(not versions.DJANGO_1_6)
        self.assertTrue(versions.DJANGO_1_7)
        self.assertTrue(not versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)

        # Less than or equal matching
        self.assertTrue(not versions.DJANGO_LTE_1_4)
        self.assertTrue(not versions.DJANGO_LTE_1_5)
        self.assertTrue(not versions.DJANGO_LTE_1_6)
        self.assertTrue(versions.DJANGO_LTE_1_7)
        self.assertTrue(versions.DJANGO_LTE_1_8)
        self.assertTrue(versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(versions.DJANGO_GTE_1_5)
        self.assertTrue(versions.DJANGO_GTE_1_6)
        self.assertTrue(versions.DJANGO_GTE_1_7)
        self.assertTrue(not versions.DJANGO_GTE_1_8)
        self.assertTrue(not versions.DJANGO_GTE_1_9)

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.8.a1'))
    def test_django_1_8_a1(self):
        """
        Tests as if we were using Django==1.8.a1.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(not versions.DJANGO_1_4)
        self.assertTrue(not versions.DJANGO_1_5)
        self.assertTrue(not versions.DJANGO_1_6)
        self.assertTrue(not versions.DJANGO_1_7)
        self.assertTrue(versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)

        # Less than or equal matching
        self.assertTrue(not versions.DJANGO_LTE_1_4)
        self.assertTrue(not versions.DJANGO_LTE_1_5)
        self.assertTrue(not versions.DJANGO_LTE_1_6)
        self.assertTrue(not versions.DJANGO_LTE_1_7)
        self.assertTrue(versions.DJANGO_LTE_1_8)
        self.assertTrue(versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(versions.DJANGO_GTE_1_5)
        self.assertTrue(versions.DJANGO_GTE_1_6)
        self.assertTrue(versions.DJANGO_GTE_1_7)
        self.assertTrue(versions.DJANGO_GTE_1_8)
        self.assertTrue(not versions.DJANGO_GTE_1_9)

    @print_info
    @mock.patch('django.get_version', mock.MagicMock(return_value='1.10'))
    def test_django_1_10(self):
        """
        Tests as if we were using Django==1.10.
        """
        from nine import versions
        reload(versions)

        # Exact version matching
        self.assertTrue(not versions.DJANGO_1_4)
        self.assertTrue(not versions.DJANGO_1_5)
        self.assertTrue(not versions.DJANGO_1_6)
        self.assertTrue(not versions.DJANGO_1_7)
        self.assertTrue(not versions.DJANGO_1_8)
        self.assertTrue(not versions.DJANGO_1_9)
        self.assertTrue(versions.DJANGO_1_10)

        # Less than or equal matching
        self.assertTrue(not versions.DJANGO_LTE_1_4)
        self.assertTrue(not versions.DJANGO_LTE_1_5)
        self.assertTrue(not versions.DJANGO_LTE_1_6)
        self.assertTrue(not versions.DJANGO_LTE_1_7)
        self.assertTrue(not versions.DJANGO_LTE_1_8)
        self.assertTrue(not versions.DJANGO_LTE_1_9)

        # Greater than or equal matching
        self.assertTrue(versions.DJANGO_GTE_1_4)
        self.assertTrue(versions.DJANGO_GTE_1_5)
        self.assertTrue(versions.DJANGO_GTE_1_6)
        self.assertTrue(versions.DJANGO_GTE_1_7)
        self.assertTrue(versions.DJANGO_GTE_1_8)
        self.assertTrue(versions.DJANGO_GTE_1_9)
        self.assertTrue(versions.DJANGO_GTE_1_10)


if __name__ == "__main__":
    unittest.main()



"""
Compatibility module. Contains information about the current Django version in
use, including (LTE and GTE statements).
"""

__title__ = 'nine.versions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'LOOSE_DJANGO_VERSION', 'LOOSE_DJANGO_MINOR_VERSION',

    'DJANGO_1_4', 'DJANGO_1_5', 'DJANGO_1_6', 'DJANGO_1_7', 'DJANGO_1_8',
    'DJANGO_1_9', 'DJANGO_1_10', 'DJANGO_1_11', 'DJANGO_2_0',

    'DJANGO_GTE_1_4', 'DJANGO_GTE_1_5', 'DJANGO_GTE_1_6', 'DJANGO_GTE_1_7',
    'DJANGO_GTE_1_8', 'DJANGO_GTE_1_9', 'DJANGO_GTE_1_10', 'DJANGO_GTE_1_11',
    'DJANGO_GTE_2_0',

    'DJANGO_LTE_1_4', 'DJANGO_LTE_1_5', 'DJANGO_LTE_1_6', 'DJANGO_LTE_1_7',
    'DJANGO_LTE_1_8', 'DJANGO_LTE_1_9', 'DJANGO_LTE_1_10', 'DJANGO_LTE_1_11',
    'DJANGO_LTE_2_0',
)

from distutils.version import LooseVersion

import django

LOOSE_DJANGO_VERSION = LooseVersion(django.get_version())
LOOSE_DJANGO_MINOR_VERSION = LooseVersion(
    '.'.join([str(i) for i in LOOSE_DJANGO_VERSION.version[0:2]])
)
LOOSE_VERSION_1_4 = LooseVersion('1.4')
LOOSE_VERSION_1_5 = LooseVersion('1.5')
LOOSE_VERSION_1_6 = LooseVersion('1.6')
LOOSE_VERSION_1_7 = LooseVersion('1.7')
LOOSE_VERSION_1_8 = LooseVersion('1.8')
LOOSE_VERSION_1_9 = LooseVersion('1.9')
LOOSE_VERSION_1_10 = LooseVersion('1.10')
LOOSE_VERSION_1_11 = LooseVersion('1.11')
LOOSE_VERSION_2_0 = LooseVersion('2.0')
LOOSE_VERSION_2_1 = LooseVersion('2.1')

# Exact versions
DJANGO_1_4 = LOOSE_VERSION_1_4 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_5
DJANGO_1_5 = LOOSE_VERSION_1_5 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_6
DJANGO_1_6 = LOOSE_VERSION_1_6 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_7
DJANGO_1_7 = LOOSE_VERSION_1_7 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_8
DJANGO_1_8 = LOOSE_VERSION_1_8 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_9
DJANGO_1_9 = LOOSE_VERSION_1_9 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_10
DJANGO_1_10 = LOOSE_VERSION_1_10 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_1_11
DJANGO_2_0 = LOOSE_VERSION_2_0 <= LOOSE_DJANGO_VERSION < LOOSE_VERSION_2_1

# LTE list
DJANGO_LTE_1_4 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_4
DJANGO_LTE_1_5 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_5
DJANGO_LTE_1_6 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_6
DJANGO_LTE_1_7 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_7
DJANGO_LTE_1_8 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_8
DJANGO_LTE_1_9 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_9
DJANGO_LTE_1_10 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_1_10
DJANGO_LTE_2_0 = LOOSE_DJANGO_MINOR_VERSION <= LOOSE_VERSION_2_0

# GTE list
DJANGO_GTE_1_4 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_4
DJANGO_GTE_1_5 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_5
DJANGO_GTE_1_6 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_6
DJANGO_GTE_1_7 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_7
DJANGO_GTE_1_8 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_8
DJANGO_GTE_1_9 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_9
DJANGO_GTE_1_10 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_1_10
DJANGO_GTE_2_0 = LOOSE_DJANGO_MINOR_VERSION >= LOOSE_VERSION_2_0



from __future__ import print_function

__title__ = 'nine.tests.base'
__author__ = 'Artur Barseghyan'
__copyright__ = 'Copyright (c) 2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('PRINT_INFO', 'TRACK_TIME', 'print_info',)


PRINT_INFO = True
TRACK_TIME = False

def print_info(func):
    """
    Prints some useful info.
    """
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        if TRACK_TIME:
            import simple_timer
            timer = simple_timer.Timer() # Start timer

        result = func(self, *args, **kwargs)

        if TRACK_TIME:
            timer.stop() # Stop timer

        print('\n\n%s' % func.__name__)
        print('============================')
        if func.__doc__:
            print('""" %s """' % func.__doc__.strip())
        print('----------------------------')
        if result is not None:
            print(result)
        if TRACK_TIME:
            print('done in %s seconds' % timer.duration)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner



import sys
import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
    readme = readme.replace('.. code-block:: none', '.. code-block::')
except:
    readme = ''

version = '0.1.7'

install_requires = [
    #'Django',
    #'mock',
]

tests_require = [
    'Django',
    'mock',
]

try:
    PY2 = sys.version_info[0] == 2
    LTE_PY26 = PY2 and (7 > sys.version_info[1])
    PY3 = sys.version_info[0] == 3

    if LTE_PY26:
        install_requires.append('importlib')
except:
    pass

setup(
    name = 'django-nine',
    version = version,
    description = ("Compatibility library for Django."),
    long_description = "{0}".format(readme),
    classifiers = [
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Environment :: Web Environment",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
    ],
    keywords = 'django, compatibility',
    author = 'Artur Barseghyan',
    author_email = 'artur.barseghyan@gmail.com',
    url = 'https://github.com/barseghyanartur/django-nine/',
    package_dir = {'': 'src'},
    packages = find_packages(where='./src'),
    license = 'GPL 2.0/LGPL 2.1',
    install_requires = install_requires,
    tests_require = tests_require,
    package_data = {},
    include_package_data = True,
)
